//	phdcc.ContactForm
//	Copyright  2011 PHD Computer Consultants Ltd, PHDCC
//	http://www.phdcc.com/phdcc.ContactForm/

using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml.Serialization;

using DotNetNuke;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Services.Exceptions;
using DotNetNuke.Services.Localization;

namespace phdcc.ContactForm
{
	///////////////////////////////////////////////////////////////////////
	/// <summary>
	/// The Settings class manages Module Settings
	/// </summary>
	/// <remarks>
	/// </remarks>
	/// <history>
	/// </history>
	partial class Settings : ModuleSettingsBase
	{
		[Serializable]
		public class Category
		{
			public int CategoryNo { get; set; }
			public string Name { get; set; }
			public string Emails { get; set; }
			public bool Selected { get; set; }
			[XmlIgnore]
			public string Summary
			{
				get
				{
					string summary = Name + ": " + Emails;
					if (Selected)
						summary += " (Selected)";
					return summary;
				}
			}
		}

		public const string CAPTCHA_Simple_Always = "1";
		public const string CAPTCHA_Simple_AnonOnly = "2";

		private string Info = "Contact Form";
		private string Instructions = "Please leave a message using this contact form:";
		private bool CC = false;
		private bool ShowCC = true;
		private string HourlyThreshold = "3";
		private string CAPTCHA = CAPTCHA_Simple_AnonOnly;

		private List<Category> categories = new List<Category>();

		///////////////////////////////////////////////////////////////////////
		/// <summary>
		/// LoadSettings loads the settings from the Database and displays them
		/// </summary>
		/// <remarks>
		/// </remarks>
		/// <history>
		/// </history>
		/// -----------------------------------------------------------------------------
		public override void LoadSettings()
		{
			try
			{
				//string CSSlink = "<link rel='stylesheet' href='" + this.ControlPath + "module.css' />";
				//Page.Header.Controls.Add(new LiteralControl(CSSlink));

				if (Page.IsPostBack == false)
				{
					btnDelete.Attributes["onclick"] = "return confirm('Are you sure you want to delete this category?')";

					if (ModuleSettings["Info"] != null)
						Info = ModuleSettings["Info"] as string;
					if (ModuleSettings["Instructions"] != null)
						Instructions = ModuleSettings["Instructions"] as string;
					if (ModuleSettings["CC"] != null)
					{
						string strCC = ModuleSettings["CC"] as string;
						CC = (strCC == "True");
					}
					if (ModuleSettings["ShowCC"] != null)
					{
						string strShowCC = ModuleSettings["ShowCC"] as string;
						ShowCC = (strShowCC == "True");
					}
					if (ModuleSettings["HourlyThreshold"] != null)
						HourlyThreshold = ModuleSettings["HourlyThreshold"] as string;
					if (ModuleSettings["CAPTCHA"] != null)
						CAPTCHA = ModuleSettings["CAPTCHA"] as string;

					for (int CategoryNo = 1; ; CategoryNo++)
					{
						if (ModuleSettings["CategoryName" + CategoryNo] == null)
							break;

						Category cat = new Category();
						cat.CategoryNo = CategoryNo;
						cat.Name = ModuleSettings["CategoryName" + CategoryNo] as string;
						cat.Emails = ModuleSettings["CategoryEmails" + CategoryNo] as string;
						string strSelected = ModuleSettings["CategorySelected" + CategoryNo] as string;
						cat.Selected = (strSelected == "True");

						categories.Add(cat);
					}

					string[] HourlyThresholds = new string[] { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "20", "30", "40", "50", };
					List<string> lHourlyThresholds = new List<string>(HourlyThresholds);
					if( !lHourlyThresholds.Contains(HourlyThreshold))
						lHourlyThresholds.Add(HourlyThreshold);
					ddlHourlyThreshold.DataSource = lHourlyThresholds;
					ddlHourlyThreshold.DataBind();

					txtInfo.Text = Info;
					txtInstructions.Text = Instructions;
					chkCC.Checked = CC;
					chkShowCC.Checked = ShowCC;
					ddlHourlyThreshold.SelectedValue = HourlyThreshold.ToString();
					if (CAPTCHA=="0" || CAPTCHA=="1" || CAPTCHA=="2")
						ddlCAPTCHA.SelectedValue = CAPTCHA;


					lbCategories.DataTextField = "Summary";
					lbCategories.DataValueField = "CategoryNo";

					SaveCategories();
				}
			}
			catch (Exception exc) //Module failed to load
			{
				Exceptions.ProcessModuleLoadException(this, exc);
			}
		}

		///////////////////////////////////////////////////////////////////////
		/// <summary>
		/// UpdateSettings saves the modified settings to the Database
		/// </summary>
		/// <remarks>
		/// </remarks>
		/// <history>
		/// </history>
		public override void UpdateSettings()
		{
			try
			{
				ModuleController objModules = new ModuleController();

				objModules.UpdateModuleSetting(ModuleId, "Info", txtInfo.Text);
				objModules.UpdateModuleSetting(ModuleId, "Instructions", txtInstructions.Text);
				objModules.UpdateModuleSetting(ModuleId, "CC", chkCC.Checked.ToString());
				objModules.UpdateModuleSetting(ModuleId, "ShowCC", chkShowCC.Checked.ToString());
				objModules.UpdateModuleSetting(ModuleId, "HourlyThreshold", ddlHourlyThreshold.SelectedValue);
				objModules.UpdateModuleSetting(ModuleId, "CAPTCHA", ddlCAPTCHA.SelectedValue);

				objModules.UpdateModuleSetting(ModuleId, "Set", "true");

				categories = (List<Category>)ViewState["categories"];

				int CategoryNo = 1;
				foreach (Category cat in categories)
				{
					objModules.UpdateModuleSetting(ModuleId, "CategoryName" + CategoryNo, cat.Name);
					objModules.UpdateModuleSetting(ModuleId, "CategoryEmails" + CategoryNo, cat.Emails);
					objModules.UpdateModuleSetting(ModuleId, "CategorySelected" + CategoryNo, cat.Selected.ToString());
					CategoryNo++;
				}
				objModules.DeleteModuleSetting(ModuleId, "CategoryName" + CategoryNo);

				//refresh cache
				SynchronizeModule();
			}
			catch (Exception exc) //Module failed to load
			{
				Exceptions.ProcessModuleLoadException(this, exc);
			}
		}


		///////////////////////////////////////////////////////////////////////
		protected void Add_Click(object sender, EventArgs e)
		{
			btnAdd.Enabled = false;
			btnDelete.Enabled = false;
			btnEdit.Enabled = false;

			txtCategoryName.Text = null;
			txtEmails.Text = null;
			chkSelected.Checked = false;

			ViewState.Remove("CategoryNo");

			divCategory.Visible = true;
		}
		///////////////////////////////////////////////////////////////////////
		protected void Edit_Click(object sender, EventArgs e)
		{
			if (lbCategories.SelectedIndex < 0) return;
			btnAdd.Enabled = false;
			btnDelete.Enabled = false;
			btnEdit.Enabled = false;

			categories = (List<Category>)ViewState["categories"];

			foreach (Category cat in categories)
			{
				if (cat.CategoryNo.ToString() == lbCategories.SelectedValue)
				{
					txtCategoryName.Text = cat.Name;
					txtEmails.Text = cat.Emails;
					chkSelected.Checked = cat.Selected;
					divCategory.Visible = true;
					ViewState["CategoryNo"] = lbCategories.SelectedValue;
					break;
				}
			}
		}
		///////////////////////////////////////////////////////////////////////
		protected void Delete_Click(object sender, EventArgs e)
		{
			if (lbCategories.SelectedIndex < 0) return;

			categories = (List<Category>)ViewState["categories"];

			Category catdel = null;
			foreach (Category cat in categories)
			{
				if (cat.CategoryNo.ToString() == lbCategories.SelectedValue)
				{
					catdel = cat;
					break;
				}
			}
			if (catdel != null)
				categories.Remove(catdel);

			SaveCategories();
		}
		///////////////////////////////////////////////////////////////////////
		protected void Save_Click(object sender, EventArgs e)
		{
			string FieldMissing = Localization.GetString("FieldMissing", LocalResourceFile);	// Please enter a value in this field.
			string BadEmail = Localization.GetString("BadEmail", LocalResourceFile);	// Please enter a valid email address.

			bool Validates = true;

			string CategoryName = txtCategoryName.Text.Trim();
			if (String.IsNullOrEmpty(CategoryName))
			{
				lblCategoryNameError.Text = FieldMissing;
				Validates = false;
			}

			string Emails = txtEmails.Text.Trim();
			if (String.IsNullOrEmpty(Emails))
			{
				lblEmailsError.Text = FieldMissing;
				Validates = false;
			}
			string[] aEmails = Emails.Split(new char[] { ';' });
			Regex regex = new Regex(@"\b[a-zA-Z0-9._%-+']+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}\b");
			foreach (string email in aEmails)
			{
				if (!regex.IsMatch(email.Trim()))
				{
					lblEmailsError.Text = BadEmail;
					Validates = false;
				}
			}

			if (!Validates)
				return;

			categories = (List<Category>)ViewState["categories"];

			string strCategoryNo = ViewState["CategoryNo"] as string;

			if (strCategoryNo == null)
			{
				Category cat = new Category();
				cat.CategoryNo = categories.Count + 1;
				cat.Name = CategoryName;
				cat.Emails = Emails;
				cat.Selected = chkSelected.Checked;
				categories.Add(cat);
			}
			else
			{
				foreach (Category cat in categories)
				{
					if (cat.CategoryNo.ToString() == strCategoryNo)
					{
						cat.Name = CategoryName;
						cat.Emails = Emails;
						cat.Selected = chkSelected.Checked;
						break;
					}
				}
			}

			SaveCategories();
			divCategory.Visible = false;
		}

		///////////////////////////////////////////////////////////////////////
		private void SaveCategories()
		{
			ViewState["categories"] = categories;
			lbCategories.DataSource = categories;
			lbCategories.DataBind();
		
			btnAdd.Enabled = true;
			btnDelete.Enabled = lbCategories.Items.Count > 0;
			btnEdit.Enabled = lbCategories.Items.Count > 0;

			lblCategoryNameError.Text = null;
			lblEmailsError.Text = null;
			lblSelectedError.Text = null;
		}
		///////////////////////////////////////////////////////////////////////
	}
}


