//	phdcc.ContactForm
//	Copyright  2011 PHD Computer Consultants Ltd, PHDCC
//	http://www.phdcc.com/phdcc.ContactForm/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Web.UI;
using System.Web.UI.WebControls;

using DotNetNuke;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Entities.Modules.Actions;
using DotNetNuke.Entities.Tabs;
using DotNetNuke.Security;
using DotNetNuke.Services.Exceptions;
using DotNetNuke.Services.Localization;
using DotNetNuke.Services.Log.EventLog;
using DotNetNuke.Services.Mail;

namespace phdcc.ContactForm
{
	///////////////////////////////////////////////////////////////////////
	/// <summary>
	/// The ViewContactForm class displays the content
	/// </summary>
	/// <remarks>
	/// </remarks>
	/// <history>
	/// </history>
	partial class ViewContactForm : PortalModuleBase, IActionable
	{
		public class Category
		{
			public int CategoryNo { get; set; }
			public string Name { get; set; }
			public string Emails { get; set; }
			public bool Selected { get; set; }
		}
		///////////////////////////////////////////////////////////////////////

		public const int CAPTCHA_Simple_Always = 1;
		public const int CAPTCHA_Simple_AnonOnly = 2;

		private string Info = "Contact Form";
		private string Instructions = "Please leave a message using this contact form:";
		private bool CC = false;
		private bool ShowCC = true;
		private int HourlyThreshold = 3;
		private int CAPTCHA = CAPTCHA_Simple_AnonOnly;

		private List<Category> categories = new List<Category>();

		protected void Page_Init(System.Object sender, System.EventArgs e)
		{
			// Not needed as module.css added by default
			//string CSSlink = "<link rel='stylesheet' href='" + this.ControlPath + "ContactForm.css' />";
			//Page.Header.Controls.Add(new LiteralControl(CSSlink));

		}
		///////////////////////////////////////////////////////////////////////
		/// <summary>
		/// Page_Load runs when the control is loaded
		/// </summary>
		/// <remarks>
		/// </remarks>
		/// <history>
		/// </history>
		protected void Page_Load(System.Object sender, System.EventArgs e)
		{
			lblSentMessage.Text = null;
			lblSentMessage.ForeColor = Color.Black;
			lblError.InnerText = null;

			try
			{
				bool SettingsDefaulted = false;
				if (Settings["Set"] == null)
					SettingsDefaulted = true;
				if( Settings["Info"]!=null)
					Info = Settings["Info"] as string;
				if( Settings["Instructions"]!=null)
					Instructions = Settings["Instructions"] as string;
				if (Settings["CC"] != null)
				{
					string strCC = Settings["CC"] as string;
					CC = (strCC == "True");
				}
				if (Settings["ShowCC"] != null)
				{
					string strShowCC = Settings["ShowCC"] as string;
					ShowCC = (strShowCC == "True");
				}
				if (Settings["HourlyThreshold"] != null)
				{
					string strHourlyThreshold = Settings["HourlyThreshold"] as string;
					Int32.TryParse(strHourlyThreshold, out HourlyThreshold);
				}
				if (Settings["CAPTCHA"] != null)
				{
					string strCAPTCHA = Settings["CAPTCHA"] as string;
					Int32.TryParse(strCAPTCHA, out CAPTCHA);
				}

				for (int CategoryNo = 1; ; CategoryNo++)
				{
					if (Settings["CategoryName" + CategoryNo] == null)
						break;

					Category cat = new Category();
					cat.CategoryNo = CategoryNo;
					cat.Name = Settings["CategoryName" + CategoryNo] as string;
					cat.Emails = Settings["CategoryEmails" + CategoryNo] as string;
					string strSelected = Settings["CategorySelected" + CategoryNo] as string;
					cat.Selected = (strSelected == "True");
					categories.Add(cat);
				}

				bool CategoriesDefaulted = false;
				if (categories.Count == 0)
				{
					Category cat = new Category();
					cat.CategoryNo = -1;
					cat.Name = "Contact";
					cat.Emails = PortalSettings.Email;
					cat.Selected = true;
					categories.Add(cat);
					CategoriesDefaulted = true;
				}


				if (!Page.IsPostBack)
				{
					// Show our module name and version
					lblModuleInfo.Text = "<a href='http://www.phdcc.com/phdcc.ContactForm/'>"+
						ModuleConfiguration.DesktopModule.ModuleName + "</a> " + ModuleConfiguration.DesktopModule.Version;
					if (Request.IsAuthenticated && EditMode && IsEditable)
					{
						if (SettingsDefaulted)
						{
							divAdmin.Visible = true;
							lblDefaulted.Text = "By default contact form messages will be sent to the portal administrator: "+PortalSettings.Email;
						}
					}

					litInfo.Text = Info;
					litInstructions.Text = Instructions;
					chkCC.Checked = CC;
					divCC.Visible = ShowCC;

					divCategory.Visible = true;

					if (CategoriesDefaulted)
						divCategory.Visible = false;
					
					ddlCategory.DataSource = categories;
					ddlCategory.DataTextField = "Name";
					ddlCategory.DataValueField = "CategoryNo";
					ddlCategory.DataBind();

					// Select default category (defaults to first)
					foreach (Category cat in categories)
					{
						if (cat.Selected)
						{
							ddlCategory.SelectedValue = cat.CategoryNo.ToString();
							break;
						}
					}

					divCAPTCHA.Visible = false;
					if (CAPTCHA == CAPTCHA_Simple_Always)
					{
						divCAPTCHA.Visible = true;
					}
					if (CAPTCHA == CAPTCHA_Simple_AnonOnly && UserInfo.UserID < 0)
					{
						divCAPTCHA.Visible = true;
					}

					string FieldRequired = Localization.GetString("Required", LocalResourceFile);		// This field is required
					lblNameRequired.Attributes["title"] = FieldRequired;
					lblEmailRequired.Attributes["title"] = FieldRequired;
					lblSubjectRequired.Attributes["title"] = FieldRequired;
					lblCategoryRequired.Attributes["title"] = FieldRequired;
					lblMessageRequired.Attributes["title"] = FieldRequired;
			
					lblName.Text = Localization.GetString("Name",LocalResourceFile);							// Your name:
					lblEmail.Text = Localization.GetString("Email", LocalResourceFile);						// Your email address:
					lblSubject.Text = Localization.GetString("Subject", LocalResourceFile);				// Subject:
					lblCategory.Text = Localization.GetString("Category", LocalResourceFile);			// Category:
					lblMessage.Text = Localization.GetString("Message", LocalResourceFile);				// Message:
					lblCC.Text = Localization.GetString("CC", LocalResourceFile);									// Send me a copy:
					btnSendEmail.Text = Localization.GetString("SendEmail", LocalResourceFile);		// Send Email

					string CAPTCHA_prefix = Localization.GetString("CAPTCHA", LocalResourceFile);	// To stop spam, please enter this number:
					int CAPTCHA_value = new Random().Next(1000);
					lblCAPTCHA.Text = CAPTCHA_prefix + " " + CAPTCHA_value.ToString();
					ViewState["CAPTCHA"] = CAPTCHA_value;

					if( UserInfo.UserID>0)
					{
						txtName.Text = UserInfo.DisplayName;
						txtEmail.Text = UserInfo.Email;
					}
				}
			}
			catch (Exception exc) //Module failed to load
			{
				Exceptions.ProcessModuleLoadException(this, exc);
				lblError.InnerText = "Error in Page_Load: "+exc.Message;
				lblError.Visible = true;
			}

		}

		///////////////////////////////////////////////////////////////////////
		/// <summary>
		/// Registers the module actions required for interfacing with the portal framework
		/// </summary>
		/// <value></value>
		/// <returns></returns>
		/// <remarks></remarks>
		/// <history>
		/// </history>
		public ModuleActionCollection ModuleActions
		{
			get
			{
				ModuleActionCollection Actions = new ModuleActionCollection();
				//Actions.Add(this.GetNextActionID(), Localization.GetString(ModuleActionType.AddContent, this.LocalResourceFile), ModuleActionType.AddContent, "", "", this.EditUrl(), false, SecurityAccessLevel.Edit, true, false);
				return Actions;
			}
		}

		///////////////////////////////////////////////////////////////////////
		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected void btnSettings_Click(object sender, EventArgs e)
		{
			Response.Redirect(DotNetNuke.Common.Globals.NavigateURL(PortalSettings.ActiveTab.TabID, "Module", "moduleid", ModuleId.ToString()), false);
		}

		///////////////////////////////////////////////////////////////////////
		protected void btnSendEmail_Click(object sender, EventArgs e)
		{
			if (ViewState["SendAnother"] != null)
			{
				txtName.Enabled = true;
				txtEmail.Enabled = true;
				txtSubject.Enabled = true;
				txtMessage.Enabled = true;
				ddlCategory.Enabled = true;
				chkCC.Enabled = true;
				txtCAPTCHA.Enabled = true;
				txtSubject.Text = null;
				txtMessage.Text = null;
				btnSendEmail.Text = Localization.GetString("SendEmail", LocalResourceFile);
				ViewState.Remove("SendAnother");
				return;
			}

			lblNameError.Text = null;
			lblEmailError.Text = null;
			lblSubjectError.Text = null;
			lblCategoryError.Text = null;
			lblMessageError.Text = null;
			lblCAPTCHAError.Text = null;

			string FieldMissing = Localization.GetString("FieldMissing",LocalResourceFile);	// Please enter a value in this field.
			string NothingSelected = Localization.GetString("NothingSelected", LocalResourceFile);	// Please select a category.
			string BadEmail = Localization.GetString("BadEmail", LocalResourceFile);	// Please enter a valid email address.

			string FromName = txtName.Text.Trim();
			string FromEmail = txtEmail.Text.Trim();
			string Subject = txtSubject.Text.Trim();
			string CategoryName = "";
			string strCategoryNo = "";
			if (ddlCategory.SelectedIndex >= 0)
			{
				CategoryName = ddlCategory.SelectedItem.Text;
				strCategoryNo = ddlCategory.SelectedValue.Trim();
			}
			string Message = txtMessage.Text.Trim();
			bool CC = chkCC.Checked;
			string strCAPTCHA = txtCAPTCHA.Text.Trim();
		
			bool Validates = true;
			if( String.IsNullOrEmpty(FromName))
			{
				lblNameError.Text = FieldMissing;
				Validates = false;
			}
			if (String.IsNullOrEmpty(Subject))
			{
				lblSubjectError.Text = FieldMissing;
				Validates = false;
			}
			if (String.IsNullOrEmpty(Message))
			{
				lblMessageError.Text = FieldMissing;
				Validates = false;
			}

			if (String.IsNullOrEmpty(FromEmail))
			{
				lblEmailError.Text = FieldMissing;
				Validates = false;
			}
			else
			{
				Regex regex = new Regex(@"\b[a-zA-Z0-9._%-+']+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}\b");
				if( !regex.IsMatch(FromEmail))
				{
					lblEmailError.Text = BadEmail;
					Validates = false;
				}
			}

			bool CheckCAPTCHA = (CAPTCHA == CAPTCHA_Simple_Always);
			if ((CAPTCHA == CAPTCHA_Simple_AnonOnly) && (UserInfo.UserID < 0))
				CheckCAPTCHA = true;

			if (CheckCAPTCHA)
			{
				if (ViewState["CAPTCHA"] is int)
				{
					int CAPTCHA_value = (int)ViewState["CAPTCHA"];
					if (strCAPTCHA != CAPTCHA_value.ToString())
					{
						lblCAPTCHAError.Text = FieldMissing;
						Validates = false;
					}
				}
				else
				{
					lblCAPTCHAError.Text = FieldMissing;
					Validates = false;
				}
			}

			if (ddlCategory.SelectedIndex == -1)
			{
				lblCategoryError.Text = NothingSelected;
				Validates = false;
			}
			string Emails = null;
			foreach (Category cat in categories)
			{
				if (cat.CategoryNo.ToString() == strCategoryNo)
				{
					Emails = cat.Emails;
					break;
				}
			}
			if (Emails == null)
			{
				lblCategoryError.Text = NothingSelected;
			}


			if (!Validates) return;

			string SentMessage = Localization.GetString("SentMessage", LocalResourceFile);	// sent a message using the contact form at

			Subject = "[" + PortalSettings.PortalName + " - " + CategoryName + "] " + Subject;
			string IPaddress = Request.ServerVariables["REMOTE_ADDR"];
			string From = FromName + " <" + FromEmail + ">";

			string body = FromName+" "+SentMessage+" "+TabController.CurrentPage.FullUrl+
				"\r\n\r\n"+
				Message +
				"\r\n\r\nIP: " + IPaddress +
				"\r\nFrom: " + FromName + ": " + FromEmail;

			//EventLogController elc = new EventLogController();
			//elc.AddLog("phdcc.ContactForm", "Email sent: " + body, PortalSettings, UserId, EventLogController.EventLogType.ADMIN_ALERT);
			
			string rv = phdccSendMail(From, Emails, "", "", FromEmail, Subject, body);

			bool AnyError = false;
			string msg = Localization.GetString("SentOK", LocalResourceFile);		// Your message has been sent.
			if (rv != "")
			{
				msg = Localization.GetString("NotSentOK", LocalResourceFile) +		// Sorry, this error occurred when sending your message:
							" " + rv;
				AnyError = true;
			}
			
			if (CC)
			{
				rv = phdccSendMail(From, FromEmail, "", "", FromEmail, Subject, body);
				if (rv != "")
				{
					msg += "<br />"+Localization.GetString("CCNotSentOK", LocalResourceFile) + " " + rv;	// However, sending the copy email to you failed with this error:
					AnyError = true;
				}
			}
			if (AnyError) lblSentMessage.ForeColor = Color.Red;
			lblSentMessage.Text = msg;

			if (!AnyError)
			{
				txtName.Enabled = false;
				txtEmail.Enabled = false;
				txtSubject.Enabled = false;
				ddlCategory.Enabled = false;
				txtMessage.Enabled = false;
				chkCC.Enabled = false;
				txtCAPTCHA.Enabled = false;
				btnSendEmail.Text = Localization.GetString("SendAnother", LocalResourceFile);	// Send another message
				ViewState["SendAnother"] = true;
			}
		}

		///////////////////////////////////////////////////////////////////////
		public string phdccSendMail(string MailFrom, string MailTo, string CC, string Bcc, string ReplyTo, string Subject, string Body)
		{
			string[] Attachments = new string[0];
			string rv = Mail.SendMail(MailFrom, MailTo, CC, Bcc, ReplyTo, MailPriority.Normal, Subject, MailFormat.Text, Encoding.UTF8, Body + " ", Attachments, "", "", "", "", false);
			if (!String.IsNullOrEmpty(rv))
			{
				// Log error as Admin Alert
				EventLogController elc = new EventLogController();
				elc.AddLog("phdcc.ContactForm", "SendEmail fail: " + rv+"  Message was "+Subject+": "+Body, PortalSettings, UserId, EventLogController.EventLogType.ADMIN_ALERT);
			}
			return rv;
		}
		///////////////////////////////////////////////////////////////////////
		
	}

	///////////////////////////////////////////////////////////////////////
}

